﻿// change list
// 2018-02-05: 
// - mix horizontal/vertical grid
// - added color choice between black/white/primary/secondary
// - bug CICC in design solved

using PaintDotNet;
using PaintDotNet.Effects;

using System;
using System.Drawing;
using System.Reflection;
using System.Globalization;
using System.Runtime.InteropServices;
[assembly: AssemblyTitle("Hexagonal Grid")]
[assembly: AssemblyDescription("Tool to create hexagonal Grid.")]
[assembly: AssemblyConfiguration("hexagonal|grid|generator|honeycomb")]
[assembly: AssemblyCompany("MadJik (Jean-Claude JAY)")]
[assembly: AssemblyProduct("Hexagonal Grid Plugin")]
[assembly: AssemblyCopyright("Copyright © 2018 MadJik (Jean-Claude JAY)")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]
[assembly: ComVisible(true)]
[assembly: AssemblyVersion("4.0.0.*")]
[assembly: AssemblyFileVersion("4.0.0.0")]

namespace HexaGrid
{
  public class PluginSupportInfo : IPluginSupportInfo
  {
    public string Author { get { return "MadJik"; } }
    public string Copyright { get { return ((AssemblyCopyrightAttribute)GetType().Assembly.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false)[0]).Copyright; } }
    public string DisplayName { get { return ((AssemblyProductAttribute)GetType().Assembly.GetCustomAttributes(typeof(AssemblyProductAttribute), false)[0]).Product; } }
    public Version Version { get { return GetType().Assembly.GetName().Version; } }
    public Uri WebsiteUri { get { return new Uri("http://forums.getpaint.net/index.php?showtopic=4279"); } }
    public string plugin_browser_Keywords
    {
      get
      {
        switch (CultureInfo.CurrentUICulture.Name)
        {
          case "fr": return "outil|générateur|grille|nid|abeille|hexagonale";
          case "ru": return "ru|шестиугольная|сетка|генератор|honeycomb";
          default: return "hexagonal|grid|generator|honeycomb";
        }
      }
    }
    public string plugin_browser_Description
    {
      get
      {
        switch (CultureInfo.CurrentUICulture.Name)
        {
          case "fr": return "Outil de création de grilles hexagonales (nid d'abeille).";
          case "ru": return "Инструмент для создания шестиугольных сеток (ячеек).";
          default: return "Tool for creating hexagonal grids (honeycomb).";
        }
      }
    }
  }

  [PluginSupportInfo(typeof(PluginSupportInfo), DisplayName = "Hexagonal Grid")]
  public class EffectPlugin : Effect
  {
    public static string CICC = CultureInfo.CurrentUICulture.EnglishName;

    public static string StaticName { get { if (CICC == "French") return "Grille hexagonale"; else if (CICC == "Russian") return "Шестиугольная сетка"; else return "Hexagonal Grid"; } }
    public static string StaticDescription
    {
      get
      {
        switch (CultureInfo.CurrentUICulture.Name)
        {
          case "fr": return "Outil de création de grilles hexagonales (nid d'abeille).";
          case "ru": return "Инструмент для создания шестиугольных сеток (ячеек).";
          default: return "Tool for creating hexagonal grids (honeycomb).";
        }
      }
    }
    public static string StaticCopyright
    {
      get
      {
        if (CICC == "French") return "Copyright © 2018 par MadJik\nTous droits réservés.";
        else if (CICC == "Russian") return "Copyright ©MadJik, 2018\nВсе права защищены.";
        else return "Copyright © 2018 by MadJik\nAll rights reserved.";
      }
    }
    public static string StaticSubMenuName { get { return SubmenuNames.Render; } }

    public static Bitmap StaticIcon { get { return new Bitmap(typeof(EffectPlugin), "EffectPluginIcon.png"); } }    // Here is the icon

    public EffectPlugin() : base(StaticName, StaticIcon, StaticSubMenuName, EffectFlags.Configurable) { }

    public override EffectConfigDialog CreateConfigDialog()
    {
      return new EffectPluginConfigDialog();
    }

    /*
          protected override void OnCustomizeConfigUIWindowProperties(PropertyCollection props)
        {
          // Add help button to effect UI
          props[ControlInfoPropertyNames.WindowHelpContentType].Value = WindowHelpContentType.PlainText;
          var veVersion = Assembly.GetExecutingAssembly().GetName().Version;
          string strVersion = " V" + veVersion.Major + "." + veVersion.Minor + "." + veVersion.Build + "." + veVersion.Revision + "\n\n" + StaticDescription + "\n";
          props[ControlInfoPropertyNames.WindowTitle].Value = StaticName;
          props[ControlInfoPropertyNames.WindowHelpContent].Value = StaticName + strVersion + "\n" + StaticCopyright;
          base.OnCustomizeConfigUIWindowProperties(props);
        }
    */
    // OK, here's the meat of the project, the render function
    public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
    {
      // setup access to our token (UI) variables
      EffectPluginConfigToken token = (EffectPluginConfigToken)parameters;
      PdnRegion selectionRegion = EnvironmentParameters.GetSelection(srcArgs.Bounds);
      Rectangle selection = EnvironmentParameters.GetSelection(srcArgs.Bounds).GetBoundsInt();

      ColorBgra HorzColor = token.Color1;
      ColorBgra VertColor = token.Color2;
      ColorBgra SpotColor = token.ColorSpot;

      HorzColor.A = (byte)token.TransAlpha;
      VertColor.A = (byte)token.TransAlpha;
      SpotColor.A = (byte)token.TransAlpha;

      int bw = token.BrushWidth;
      Pen myPenH = new Pen(Color.FromArgb(HorzColor.A, HorzColor.R, HorzColor.G, HorzColor.B), bw);
      Pen myPenV = new Pen(Color.FromArgb(VertColor.A, VertColor.R, VertColor.G, VertColor.B), bw);
      if (bw>2) bw--;
      int bw2 = bw * 2;
      //Pen myPenW = new Pen(Color.White, 1);
      SolidBrush sc = new SolidBrush(SpotColor);
      if (token.EndMode)
      {
        myPenH.StartCap = System.Drawing.Drawing2D.LineCap.Flat;
        myPenH.EndCap = System.Drawing.Drawing2D.LineCap.Flat;
        myPenV.StartCap = System.Drawing.Drawing2D.LineCap.Flat;
        myPenV.EndCap = System.Drawing.Drawing2D.LineCap.Flat;
      }
      else
      {
        myPenH.StartCap = System.Drawing.Drawing2D.LineCap.Round;
        myPenH.EndCap = System.Drawing.Drawing2D.LineCap.Round;
        myPenV.StartCap = System.Drawing.Drawing2D.LineCap.Round;
        myPenV.EndCap = System.Drawing.Drawing2D.LineCap.Round;
      }

      long CenterX = (selection.Right - selection.Left) / 2 + selection.Left;
      long CenterY = (selection.Bottom - selection.Top) / 2 + selection.Top;
      int rayon = token.SizeHexa;
      int rayonb = token.SizeHexb;

      bool drmode1 = token.DrawMode1;
      bool drmode2 = token.DrawMode2;
      bool drmode3 = token.DrawMode3;
      bool drmode4 = token.DrawMode4;
      bool drmode5 = token.DrawMode5;
      bool drmode6 = token.DrawMode6;
      bool drsc = token.SpotCross;

      float X1 = 0, Y1 = 0, X2 = 0, Y2 = 0;
      float RhxFh, RhyFh, Rxh, Ryh, Rhxh, Rhyh;
      float RhxFv, RhyFv, Rxv, Ryv, Rhxv, Rhyv;

      // Hexagons with horizontal base and top sides
      RhxFh = (float)(rayon * Math.Cos(Math.PI / 3.0));
      RhyFh = (float)(rayonb * Math.Sin(Math.PI / 3.0));
      Rxh = 6f * RhxFh;
      Ryh = 2f * RhyFh;
      Rhxh = Rxh / 6f;
      Rhyh = Ryh / 2f;

      // Hexagons with vertical left and right sides
      RhxFv = (float)(rayon * Math.Sin(Math.PI / 3.0));
      RhyFv = (float)(rayonb * Math.Cos(Math.PI / 3.0));
      Rxv = 2f * RhxFv;
      Ryv = 6f * RhyFv;
      Rhxv = Rxv / 2f;
      Rhyv = Ryv / 6f;

      int NbXh = (int)(CenterX / Rxh) + 2;
      int NbYh = (int)(CenterY / Ryh) + 2;
      int NbXv = (int)(CenterX / Rxv) + 2;
      int NbYv = (int)(CenterY / Ryv) + 2;

      float RxD1h = 0;
      //float RyD1h = 0;
      float RxD2h = 4f * RhxFh;
      float RyD2h = 4f * RhyFh;
      float RxD3h = 2f * RhxFh;
      float RyD3h = 2f * RhyFh;

      //float RxD1v = 0;
      float RyD1v = 0;
      float RxD2v = 4f * RhxFv;
      float RyD2v = 4f * RhyFv;
      float RxD3v = 2f * RhxFv;
      float RyD3v = 2f * RhyFv;

      // The lock statement allows us GDI+ access to the drawing surface
      lock (dstArgs.Graphics)
      {
        for (int i = startIndex; i < startIndex + length; ++i)
        {
          Rectangle rect = rois[i];

          for (int y = rect.Top; y < rect.Bottom; ++y)
          {
            if (IsCancelRequested) return;
            for (int x = rect.Left; x < rect.Right; ++x)
              dstArgs.Surface[x, y] = srcArgs.Surface[x, y];
          }

          // Anti-aliasing (thanks to www.boltbait.com for this code)
          if (token.AntiAliased)
          {
            dstArgs.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
          }
          else
          {
            dstArgs.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;
          }

          // Make sure we don't draw outside of our current rectangle of intrest
          dstArgs.Graphics.Clip = new Region(rect);

          if (drmode1 || drmode2 || drmode3)
          {
            for (int y = -NbYh; y < NbYh; y++)
            {
              if (IsCancelRequested) return;
              for (int x = -NbXh; x < NbXh; x++)
              {
                if (drmode1)
                {
                  // Draw segment 1
                  X1 = x * Rxh + CenterX + RxD1h;
                  Y1 = y * Ryh + CenterY;
                  X2 = X1 + Rhxh;
                  Y2 = Y1 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 2
                  Y1 = Y2 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 3
                  X1 = X2 + 2 * Rhxh;
                  Y1 = Y2;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 4
                  X2 = X1 + Rhxh;
                  Y2 = Y1 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 5
                  Y2 = Y1 - Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 6
                  X1 = X2 + 2 * Rhxh;
                  Y1 = Y2;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);
                }
                if (drmode2)
                {
                  // Draw segment 1
                  X1 = x * Rxh + CenterX + RxD2h;
                  Y1 = y * Ryh + CenterY;
                  X2 = X1 + Rhxh;
                  Y2 = Y1 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 2
                  Y1 = Y2 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 3
                  X1 = X2 + 2 * Rhxh;
                  Y1 = Y2;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 4
                  X2 = X1 + Rhxh;
                  Y2 = Y1 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 5
                  Y2 = Y1 - Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 6
                  X1 = X2 + 2 * Rhxh;
                  Y1 = Y2;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);
                }
                if (drmode3)
                {
                  // Draw segment 1
                  X1 = x * Rxh + CenterX + RxD3h;
                  Y1 = y * Ryh + CenterY;
                  X2 = X1 + Rhxh;
                  Y2 = Y1 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 2
                  Y1 = Y2 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 3
                  X1 = X2 + 2 * Rhxh;
                  Y1 = Y2;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 4
                  X2 = X1 + Rhxh;
                  Y2 = Y1 + Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 5
                  Y2 = Y1 - Rhyh;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);

                  // Draw segment 6
                  X1 = X2 + 2 * Rhxh;
                  Y1 = Y2;
                  dstArgs.Graphics.DrawLine(myPenH, X1, Y1, X2, Y2);
                }
              }
            }
          }
          if (drmode4 || drmode5 || drmode6)
          {
            for (int y = -NbYv; y < NbYv; y++)
            {
              if (IsCancelRequested) return;
              for (int x = -NbXv; x < NbXv; x++)
              {
                if (drmode4)
                {
                  // Draw segment 1
                  X1 = x * Rxv + CenterX;
                  Y1 = y * Ryv + CenterY + RyD1v;
                  X2 = X1 + Rhxv;
                  Y2 = Y1 + Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 2
                  X1 = X2 + Rhxv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 3
                  X1 = X2;
                  Y1 = Y2 + 2 * Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 4
                  X2 = X1 + Rhxv;
                  Y2 = Y1 + Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 5
                  X2 = X1 - Rhxv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 6
                  X1 = X2;
                  Y1 = Y2 + 2 * Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);
                }
                if (drmode5)
                {
                  // Draw segment 1
                  X1 = x * Rxv + CenterX;
                  Y1 = y * Ryv + CenterY + RyD2v;
                  X2 = X1 + Rhxv;
                  Y2 = Y1 + Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 2
                  X1 = X2 + Rhxv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 3
                  X1 = X2;
                  Y1 = Y2 + 2 * Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 4
                  X2 = X1 + Rhxv;
                  Y2 = Y1 + Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 5
                  X2 = X1 - Rhxv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 6
                  X1 = X2;
                  Y1 = Y2 + 2 * Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);
                }
                if (drmode6)
                {
                  // Draw segment 1
                  X1 = x * Rxv + CenterX;
                  Y1 = y * Ryv + CenterY + RyD3v;
                  X2 = X1 + Rhxv;
                  Y2 = Y1 + Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 2
                  X1 = X2 + Rhxv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 3
                  X1 = X2;
                  Y1 = Y2 + 2 * Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 4
                  X2 = X1 + Rhxv;
                  Y2 = Y1 + Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 5
                  X2 = X1 - Rhxv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);

                  // Draw segment 6
                  X1 = X2;
                  Y1 = Y2 + 2 * Rhyv;
                  dstArgs.Graphics.DrawLine(myPenV, X1, Y1, X2, Y2);
                }
              }
            }
          }
          if (drsc)
          {
            int sbw = bw / 2;
            int sbw2 = bw2 / 2;
            if (drmode1 || drmode2 || drmode3)
            {
              for (int y = -NbYh; y < NbYh; y++)
              {
                if (IsCancelRequested) return;
                for (int x = -NbXh; x < NbXh; x++)
                {
                  if (drmode1)
                  {
                    // Draw segment 1
                    X1 = x * Rxh + CenterX + RxD1h;
                    Y1 = y * Ryh + CenterY;
                    X2 = X1 + Rhxh;
                    Y2 = Y1 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 2
                    Y1 = Y2 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 3
                    X1 = X2 + 2 * Rhxh;
                    Y1 = Y2;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 4
                    X2 = X1 + Rhxh;
                    Y2 = Y1 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 5
                    Y2 = Y1 - Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 6
                    X1 = X2 + 2 * Rhxh;
                    Y1 = Y2;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);
                  }
                  if (drmode2)
                  {
                    // Draw segment 1
                    X1 = x * Rxh + CenterX + RxD2h;
                    Y1 = y * Ryh + CenterY;
                    X2 = X1 + Rhxh;
                    Y2 = Y1 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 2
                    Y1 = Y2 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 3
                    X1 = X2 + 2 * Rhxh;
                    Y1 = Y2;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 4
                    X2 = X1 + Rhxh;
                    Y2 = Y1 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 5
                    Y2 = Y1 - Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 6
                    X1 = X2 + 2 * Rhxh;
                    Y1 = Y2;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);
                  }
                  if (drmode3)
                  {
                    // Draw segment 1
                    X1 = x * Rxh + CenterX + RxD3h;
                    Y1 = y * Ryh + CenterY;
                    X2 = X1 + Rhxh;
                    Y2 = Y1 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 2
                    Y1 = Y2 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 3
                    X1 = X2 + 2 * Rhxh;
                    Y1 = Y2;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 4
                    X2 = X1 + Rhxh;
                    Y2 = Y1 + Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 5
                    Y2 = Y1 - Rhyh;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 6
                    X1 = X2 + 2 * Rhxh;
                    Y1 = Y2;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);
                  }
                }
              }
            }
            if (drmode4 || drmode5 || drmode6)
            {
              for (int y = -NbYv; y < NbYv; y++)
              {
                if (IsCancelRequested) return;
                for (int x = -NbXv; x < NbXv; x++)
                {
                  if (drmode4)
                  {
                    // Draw segment 1
                    X1 = x * Rxv + CenterX;
                    Y1 = y * Ryv + CenterY + RyD1v;
                    X2 = X1 + Rhxv;
                    Y2 = Y1 + Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 2
                    X1 = X2 + Rhxv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 3
                    X1 = X2;
                    Y1 = Y2 + 2 * Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 4
                    X2 = X1 + Rhxv;
                    Y2 = Y1 + Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 5
                    X2 = X1 - Rhxv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 6
                    X1 = X2;
                    Y1 = Y2 + 2 * Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);
                  }
                  if (drmode5)
                  {
                    // Draw segment 1
                    X1 = x * Rxv + CenterX;
                    Y1 = y * Ryv + CenterY + RyD2v;
                    X2 = X1 + Rhxv;
                    Y2 = Y1 + Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 2
                    X1 = X2 + Rhxv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 3
                    X1 = X2;
                    Y1 = Y2 + 2 * Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 4
                    X2 = X1 + Rhxv;
                    Y2 = Y1 + Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 5
                    X2 = X1 - Rhxv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 6
                    X1 = X2;
                    Y1 = Y2 + 2 * Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);
                  }
                  if (drmode6)
                  {
                    // Draw segment 1
                    X1 = x * Rxv + CenterX;
                    Y1 = y * Ryv + CenterY + RyD3v;
                    X2 = X1 + Rhxv;
                    Y2 = Y1 + Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 2
                    X1 = X2 + Rhxv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 3
                    X1 = X2;
                    Y1 = Y2 + 2 * Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 4
                    X2 = X1 + Rhxv;
                    Y2 = Y1 + Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 5
                    X2 = X1 - Rhxv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);

                    // Draw segment 6
                    X1 = X2;
                    Y1 = Y2 + 2 * Rhyv;
                    dstArgs.Graphics.FillEllipse(sc, X1 - sbw, Y1 - sbw, sbw2, sbw2);
                    dstArgs.Graphics.FillEllipse(sc, X2 - sbw, Y2 - sbw, sbw2, sbw2);
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
